﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Cosmos.Core;

namespace RevoluionOS.Core.Desktop
{
    class Mouse
    {
        public static Cosmos.Hardware.Mouse cMouse = new Cosmos.Hardware.Mouse();
        public static Core.Resolution.Resolution Resolution = new Resolution.Resolution();
        public static Mouse Mousemouse = new Mouse();
        public static string MouseType = "Default";


        public static void Draw()
        {
            int x = Mousex();
            int y = Mousey();

            if (MouseType == "Default")
            {
                Core.Graphics.Formsforuse.Forms.DrawBlackWhite(Core.Graphics.Cursor.NormalCursor, 8, 12, (uint)x, (uint)y);//draws the mosue

            }
            else if (MouseType == "NoToActa")
            {
                Core.Graphics.Font.Ownfont7x8.ConvertText("ACTA", (uint)x - 1, (uint)y + 9, 0xffffff);//writes "ACTA"
                Core.Graphics.Formsforuse.Forms.DrawArray(Core.Desktop.Style.Mouse.NoToActa.NoActaMouse, 27, 27, (uint)x, (uint)y, 0xCD5C5C);//draws the NoToActa mouse
            }
        }



        //Show a cursor
        public static void Start()
        {
            Initialize(); //jumps to Initialize()
            //InitializeCosmosMouse();//jumps to InitializeCosmosMouse
        

            //get x and y
            int x = Mousex();
            int y = Mousey();

        }


        //starts the mouse 
        public static void InitializeCosmosMouse()
        {
            cMouse.Initialize();//will start the mouse
        }
        

        //Will show the current x location of the mouse
        public static int Mousex()
        {
            /*int x;
            x = cMouse.X;//gets the current x-coordinate of the mouse
             */

            return X;//returns x to int for Mousex
        }

        //Will show the current y location of the mouse
        public static int Mousey()
        {
            /*int y;
            y = cMouse.Y;//gets the current y-coordinate
             */

            return Y;//returns y to int for Mousey
        }


        #region " Mouse Driver "
        /// <summary>
        /// The X location of the mouse.
        /// </summary>
        public static int X;
        /// <summary>
        /// The Y location of the mouse.
        /// </summary>
        public static int Y;
        /// <summary>
        /// The state the mouse is currently in.
        /// </summary>
        public  static MouseState Buttons;

        /// <summary>
        /// This is the required call to start
        /// the mouse receiving interrupts.
        /// </summary>
        public static void Initialize()
        {
            ////enable mouse
            WaitSignal();
            BaseIOGroups.Mouse.p64.Byte = (byte)0xA8;

            //// enable interrupt
            WaitSignal();
            BaseIOGroups.Mouse.p64.Byte = (byte)0x20;
            WaitData();
            //byte status1 = (byte)(BaseIOGroups.Mouse.p60.Byte);
            byte status = (byte)(BaseIOGroups.Mouse.p60.Byte | 2);
            WaitSignal();
            BaseIOGroups.Mouse.p64.Byte = (byte)0x60;
            WaitSignal();
            BaseIOGroups.Mouse.p60.Byte = (byte)status;

            ////default 
            Write(0xF6);
            Read();  //Acknowledge

            ////Enable the mouse
            Write(0xF4);
            Read();  //Acknowledge

            //Set the IRQ 12, to the method HandleMouse
            INTs.SetIrqHandler(12, HandleMouse);
        }

        private static byte Read()
        {
            WaitData();
            return BaseIOGroups.Mouse.p60.Byte;
        }

        private static void Write(byte b)
        {
            WaitSignal();
            BaseIOGroups.Mouse.p64.Byte = 0xD4;
            WaitSignal();
            BaseIOGroups.Mouse.p60.Byte = b;
        }

        private static void WaitData()
        {
            for (int i = 0; i < 100 & ((BaseIOGroups.Mouse.p64.Byte & 1) == 1); i++)
                ;
        }

        private static void WaitSignal()
        {
            for (int i = 0; i < 100 & ((BaseIOGroups.Mouse.p64.Byte & 2) != 0); i++)
                ;
        }

        /// <summary>
        /// The possible states of a mouse.
        /// </summary>
        public enum MouseState
        {
            /// <summary>
            /// No button is pressed.
            /// </summary>
            None = 0,
            /// <summary>
            /// The left mouse button is pressed.
            /// </summary>
            Left = 1,
            /// <summary>
            /// The right mouse button is pressed.
            /// </summary>
            Right = 2,
            /// <summary>
            /// The middle mouse button is pressed.
            /// </summary>
            Middle = 4
        }


        private static byte[] mouse_byte = new byte[4];
        private static byte mouse_cycle = 0;

        public static void HandleMouse(ref INTs.IRQContext context)
        {
            switch (mouse_cycle)
            {
                case 0:
                    mouse_byte[0] = Read();

                    //Bit 3 of byte 0 is 1, then we have a good package
                    if ((mouse_byte[0] & 0x8) == 0x8)
                        mouse_cycle++;

                    break;
                case 1:
                    mouse_byte[1] = Read();
                    mouse_cycle++;
                    break;
                case 2:
                    mouse_byte[2] = Read();
                    mouse_cycle = 0;

                    if ((mouse_byte[0] & 0x10) == 0x10)
                        X -= (mouse_byte[1] ^ 0xff);
                    else
                        X += mouse_byte[1];

                    if ((mouse_byte[0] & 0x20) == 0x20)
                        Y += (mouse_byte[2] ^ 0xff);
                    else
                        Y -= mouse_byte[2];

                    if (Y < 0)
                        Y = 0;
                    if (X < 0)
                        X = 0;

                    Buttons = (MouseState)(mouse_byte[0] & 0x7);

                    break;
            }

        }
        #endregion

        public static string CheckClick()
        {

            string Click = "none";
            //if (cMouse.Buttons == Cosmos.Hardware.Mouse.MouseState.Left) Click = "left";//checks for leftclicks

            if (Buttons == MouseState.Left) Click = "left";

            //else if (cMouse.Buttons == Cosmos.Hardware.Mouse.MouseState.Right) Click = "right";//checks for rightclicks
            else if (Buttons == MouseState.Right) Click = "right";

            //else if (cMouse.Buttons == Cosmos.Hardware.Mouse.MouseState.Middle) Click = "middle";//checks for middle clicks
            else if (Buttons == MouseState.Middle) Click = "middle";

            //else if (cMouse.Buttons == Cosmos.Hardware.Mouse.MouseState.None) Click = "none";
            else if (Buttons == MouseState.None) Click = "none";

            else Click = "none";

            return Click;//returns the string click
        }

        public static Boolean LeftClick(int x, int y, int width, int height)
        {
            CheckClick();
            if (Mousex() >= x && Mousey() >= y && Mousex() <= x + width && Mousey() <= y + height && CheckClick() == "left")
            {
                return true;//returns true
            }
            else return false;//returns false
        }

        public static Boolean RightClick(int x, int y, int width, int height)
        {
            if (Mousex() >= x && Mousey() >= y && Mousex() <= x + width && Mousey() <= y + height && CheckClick() == "right")
            {
                return true;//returns true
            }
            else return false;//returns false
        }

        public static Boolean MiddleClick(int x, int y, int width, int height)
        {
            if (Mousex() >= x && Mousey() >= y && Mousex() <= x + width && Mousey() <= y + height && CheckClick() == "middle")
            {
                return true;//returns true
            }
            else return false;//returns false
        }

        public static void Clicks()
        {
            if (LeftClick(Resolution.width - 53, 0, 55, (int)Core.Graphics.Statusbar.Statusbar.Statusbarheight) == true)//opens the "clock"
            {
                Apps.ClockPopup.Click = true;
                Apps.ClockPopup.Open();
            }

            else if (LeftClick(((int)Core.Graphics.Statusbar.Statusbar.Statusbarheight - 10) / 2, ((int)Core.Graphics.Statusbar.Statusbar.Statusbarheight - 10) / 2, 10, 10) == true)//opens the statusbarPopup
            {
                Core.Graphics.Statusbar.Statusbar.PopupOpen = true;
                Core.Graphics.Statusbar.Statusbar.StatusBarPopUp();
            }


            if (Core.Desktop.Apps.Settings.Settings.SettingsOpen == true)
            {
                Core.Desktop.Apps.Settings.Settings.Clicks();//jumps to clicks in Settings app
            }
            else if (Core.Desktop.Apps.About.AboutOpen == true)
            {
                Core.Desktop.Apps.About.Clicks();//jumps to Clicks in About app
            }



            if (Core.Graphics.Statusbar.Statusbar.PopupOpen == true)
            {
                Core.Graphics.Statusbar.Statusbar.Clicks();//jumps to clicks in Statusbar Popup
            }
        }
    }
}
