﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace RevoluionOS.Core.Resolution.Driver
{
    class Screen
    {

        //Wrote by: Matt, for the PearOs team.
        private static Driver CurrentDriver;
        public static Core.Resolution.Resolution Resolution = new Resolution();
        public static Resolutions CurrentRes;
        public static int Width = 0;
        public static int Height = 0;

        /// <summary>
        /// Lists the resolutions available
        /// </summary>
        public enum Resolutions
        {
            Resolution1_320 = 1,
            Resolution2_640 = 2,
            Resolution3_800 = 3,
            Resolution4_1024 = 4,
            Resolution5_1280 = 5,
            Resolution6_1600 = 6
        };

        public enum Driver
        {
            CosmosVGA = 1,
            VMWareSVGA = 2
        };
        /// <summary>
        /// Selects the driver for use
        /// </summary>
        /// <param name="driver">The driver to use of type Screen.Driver</param>
        public static void SetDriver(Driver driver)
        {
            CurrentDriver = driver;
        }
        /// <summary>
        /// Set the resolution
        /// </summary>
        /// <param name="res">The resolution to use of type Screen.Resolutions</param>
        public static void SetMode(Resolutions res)
        {
            CurrentRes = res;
            if (CurrentRes == Screen.Resolutions.Resolution1_320)
            {
                Width = 320;
                Height = 200;
            }
            else if (CurrentRes == Screen.Resolutions.Resolution2_640)
            {
                Width = 640;
                Height = 400;
            }
            else if (CurrentRes == Screen.Resolutions.Resolution3_800)
            {
                Width = 800;
                Height = 600;
            }
            else if (CurrentRes == Screen.Resolutions.Resolution4_1024)
            {
                Width = 1024;
                Height = 768;
            }
            else if (CurrentRes == Screen.Resolutions.Resolution5_1280)
            {
                Width = 1280;
                Height = 1024;
            }
            else if (CurrentRes == Screen.Resolutions.Resolution6_1600)
            {
                Width = 1600;
                Height = 1200;
            }
            if (CurrentDriver == Driver.VMWareSVGA)
            {
                VMware_use.SetMode(res);
            }
            else if (CurrentDriver == Driver.CosmosVGA)
            {
                VMware_use.SetMode(res);
            }

            Resolution.width = 800;
            Resolution.height = Height;
            Resolution.uwidth = (uint)800;
            Resolution.uheight = (uint)Height;
        }

        public static void SetPixel(int x, int y, int Color)
        {
            if (CurrentDriver == Driver.VMWareSVGA)
            {
                VMware_use.SetPixel(x, y, Color);
            }
            else if (CurrentDriver == Driver.CosmosVGA)
            {
                VMware_use.SetPixel(x, y, Color);
            }
        }

        public static void Clear(int Color)
        {
            if (CurrentDriver == Driver.VMWareSVGA)
            {
                DrawRectangle(0, 0, Width, Height, Color);
            }
            else if (CurrentDriver == Driver.CosmosVGA)
            {
                VMware_use.Clear(Color);
            }
        }

        public static void Update()
        {
            if (CurrentDriver == Driver.VMWareSVGA)
            {
                VMware_use.Update();
            }
            else if (CurrentDriver == Driver.CosmosVGA)
            {
                VMware_use.Update();
            }
        }

        #region " Pre Defined "
        private static int i = 0;
        private static int t = 0;
        private static int count = 0;
        private static int bb, cc;
        #endregion
        public static void DrawFrame(uint[] Arr, int width, int length, int xpixel, int ypixel)
        {
            count = 0;
            for (i = 0; i < length; i++)
            {
                for (t = 0; t < width; t++, count++)
                {
                    if (Arr[count] == 0xFF00FF)
                    {
                    }
                    else
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), (int)Arr[count]);
                    }
                }
            }
        }
        public static void DrawFrameColor(uint[] Arr, int width, int length, int xpixel, int ypixel, int color)
        {
            count = 0;
            for (i = 0; i < length; i++)
            {
                for (t = 0; t < width; t++, count++)
                {
                    if (Arr[count] == 1)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color);
                    }
                }
            }
        }
        public static void DrawFrame2Colors(uint[] Arr, int width, int length, int xpixel, int ypixel, int color1, int color2)
        {
            count = 0;
            for (i = 0; i < length; i++)
            {
                for (t = 0; t < width; t++, count++)
                {
                    if (Arr[count] == 1)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color1);
                    }
                    else if (Arr[count] == 2)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color2);
                    }
                }
            }
        }
        public static void DrawFrame3Colors(uint[] Arr, int width, int length, int xpixel, int ypixel, int color1, int color2, int color3)
        {
            count = 0;
            for (i = 0; i < length; i++)
            {
                for (t = 0; t < width; t++, count++)
                {
                    if (Arr[count] == 1)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color1);
                    }
                    else if (Arr[count] == 2)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color2);
                    }
                    else if (Arr[count] == 3)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color3);
                    }
                }
            }
        }
        public static void DrawFrame4Colors(uint[] Arr, int width, int length, int xpixel, int ypixel, int color1, int color2, int color3, int color4)
        {
            count = 0;
            for (i = 0; i < length; i++)
            {
                for (t = 0; t < width; t++, count++)
                {
                    if (Arr[count] == 1)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color1);
                    }
                    else if (Arr[count] == 2)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color2);
                    }
                    else if (Arr[count] == 3)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color3);
                    }
                    else if (Arr[count] == 4)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), color4);
                    }
                }
            }
        }
        public static void DrawFrameBlackAndWhite(uint[] Arr, int width, int length, int xpixel, int ypixel)
        {
            count = 0;
            for (i = 0; i < length; i++)
            {
                for (t = 0; t < width; t++, count++)
                {
                    if (Arr[count] == 1)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), 0x000000);
                    }
                    if (Arr[count] == 2)
                    {
                        SetPixel((int)(0 + (uint)t + 0 + (uint)xpixel), (int)(0 + (uint)i + 0 + (uint)ypixel), 0xffffff);
                    }
                }
            }
        }
        public static void DrawRectangle(int x, int y, int width, int length, int color)
        {
            for (bb = x; bb < x + width; bb++)
            {
                for (cc = y; cc < y + length; cc++)
                {
                    SetPixel(bb, cc, color);
                }
            }
        }
    }
}
